# EnSimpleStaging  |  Copyright(C), 2004, Enfold Systems, LLC
# see LICENSE.txt for details

from OFS.ObjectManager import ObjectManager
from OFS.CopySupport import CopySource

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CatalogTool import CatalogTool
from Products.CMFPlone.PloneFolder import OrderedContainer

from Products.Archetypes.exceptions import ReferenceException
from Products.Archetypes.utils import shasattr
from Products.Archetypes.ReferenceEngine import Reference
from Products.Archetypes.OrderedBaseFolder \
     import OrderedContainer as ATOrderedContainer

from Products.EnSimpleStaging.config import logger

def log(msg):
    logger.info(msg)

PATTERN = '__EnSimpleStaging_%s__'
def call(self, __name__, *args, **kw):
    return getattr(self, PATTERN % __name__)(*args, **kw)

WRAPPER = '__ensimplestaging_is_wrapper_method__'
ORIG_NAME = '__ensimplestaging_original_method_name__'
def isWrapperMethod(meth):
    return getattr(meth, WRAPPER, False)

def wrap_method(klass, name, method, pattern=PATTERN):
    old_method = getattr(klass, name)
    if isWrapperMethod(old_method):
        # Double-wrapping considered harmful.
        log('Already wrapped method %s.%s, skipping.' %
            (klass.__name__, name))
        return
    log('Wrapping method %s.%s' % (klass.__name__, name))
    new_name = pattern % name
    setattr(klass, new_name, old_method)
    setattr(method, ORIG_NAME, new_name)
    setattr(method, WRAPPER, True)
    setattr(klass, name, method)

def unwrap_method(klass, name):
    old_method = getattr(klass, name)
    if not isWrapperMethod(old_method):
        raise ValueError, ('Trying to unwrap non-wrapped '
                           'method %s.%s' % (klass.__name__, name))
    orig_name = getattr(old_method, ORIG_NAME)
    new_method = getattr(klass, orig_name)
    delattr(klass, orig_name)
    setattr(klass, name, new_method)

def maybeReindex(obj):
    if isinstance(obj, CatalogTool):
        # Ignore as it has a different API for reindexObject
        return
    if getattr(obj, 'ESS_INSTALLING_LOCAL_TOOLS', False):
        # see Install.py, setup_local_tools()
        # catalogs are not yet in place
        return
    if shasattr(obj, 'notifyModified'):
        obj.notifyModified()
    if shasattr(obj, 'reindexObject'):
        obj.reindexObject(idxs=['modified'])

def delHook(self, tool, sourceObject=None, targetObject=None):
    wt = getToolByName(self, 'portal_workspaces')
    if ((sourceObject and wt._isProtectedUID(sourceObject.UID())) or
        (targetObject and wt._isProtectedUID(targetObject.UID()))):
        raise ReferenceException('This UID will come back on deployment')
    return call(self, 'delHook', tool, sourceObject, targetObject)

def _setOb(self, *args, **kw):
    ret = call(self, '_setOb', *args, **kw)
    maybeReindex(self)
    return ret

def _delOb(self, *args, **kw):
    ret = call(self, '_delOb', *args, **kw)
    maybeReindex(self)
    return ret

def _postCopy(self, *args, **kw):
    ret = call(self, '_postCopy', *args, **kw)
    maybeReindex(self)
    return ret

def moveObject(self, *args, **kw):
    ret = call(self, 'moveObject', *args, **kw)
    maybeReindex(self)
    return ret

def moveObjectsByDelta(self, *args, **kw):
    ret = call(self, 'moveObjectsByDelta', *args, **kw)
    maybeReindex(self)
    return ret

wrap_method(Reference, 'delHook',
                delHook, pattern=PATTERN)

wrap_method(ObjectManager, '_setOb',
            _setOb, pattern=PATTERN)

wrap_method(ObjectManager, '_delOb',
            _delOb, pattern=PATTERN)

wrap_method(CopySource, '_postCopy',
            _postCopy, pattern=PATTERN)

wrap_method(OrderedContainer, 'moveObject',
            moveObject, pattern=PATTERN)

wrap_method(OrderedContainer, 'moveObjectsByDelta',
            moveObjectsByDelta, pattern=PATTERN)

wrap_method(ATOrderedContainer, 'moveObject',
            moveObject, pattern=PATTERN)

wrap_method(ATOrderedContainer, 'moveObjectsByDelta',
            moveObjectsByDelta, pattern=PATTERN)
